module BackOffice
  module Auth
    extend ActiveSupport::Concern

    included do
      before_action :authorize
      delegate      :current_user, to: :current_device
      delegate      :authorized?,  to: :policy
      helper_method :current_user, :authorized?
    end

    private

    def current_device
      @current_device ||= Device.current(device_token, device_attributes)
    end

    def device_token
      authenticate_with_http_token { |token, _| token } || cookies.signed[:token]
    end

    def device_attributes
      {
        token:      SecureRandom.urlsafe_base64,
        user_agent: request.user_agent,
        remote_ip:  request.remote_ip
      }
    end

    def sign_in(user)
      current_device.update(user: user)
      cookies.signed[:token] = current_device.token
    end

    def sign_out
      cookies.delete(:token)
      session.clear
    end

    def authorize
      return true if authorized?(controller_name, action_name, resource)

      respond_to do |format|
        format.any(:js, :json) { head :unauthorized }
        format.html { redirect_to(root_path, alert: t(:unauthorized)) }
      end
    end

    def policy
      @policy ||= policy_class.new(current_user)
    end

    def policy_class
      ApplicationPolicy
    end

    def resource
      nil
    end
  end
end
